# "files.pl" in the Forum News Gateway
# copyright 1994,1995 by Swarthmore College

# Deal with files that store associative arrays.

# MyDBMOpen(*AssocArray,$Filename)
# Read the file into the associative array.
# The file is in the format:
#	key1
#	value1
#	key2
#	value2
#	...
# Of course, this format wouldn't work for keys or values which contain newlines.
# Returns 0 if the file did not exist (perhaps meaning that the associative array
# should be initialized), 1 if the file did exist.

sub MyDBMOpen {
	local(*AssocArray,$Filename) = @_;
	
	undef %AssocArray;	
	%AssocArray = ();
	
	if ($Filename ne "") {
		open(TEMP,"<$Filename") || return 0;
		local($OldInDelim) = $/;
		$/ = "\n";
		local($Key,$Value);
		while (($Key = <TEMP>) && ($Value = <TEMP>)) {
			chop($Key);
			chop($Value);
			$AssocArray{$Key} = $Value;
		}
		close(TEMP);
		$/ = $OldInDelim;
	}
	
	return 1;
}

# MyDBMClose(*AssocArray,$Filename)
# Write the associative array out to the file, and clear the array.
# See the comment above to MyDBMOpen for the file format.

sub MyDBMClose {
	local(*AssocArray,$Filename) = @_;
	
	&MyDBMFlush(*AssocArray,$Filename);
	
	undef %AssocArray;
	%AssocArray = ();
}

# MyDBMFlush(*AssocArray,$Filename)
# Write the associative array out to the file.
# See the comment above to MyDBMOpen for the file format.

sub MyDBMFlush {
	local(*AssocArray,$Filename) = @_;
	
	if ($Filename ne "") {
		open(TEMP,">$Filename.tmp");
		local($Key,$Value);
		while (($Key,$Value) = each %AssocArray) {
			print TEMP "$Key\n$Value\n";
		}
		close(TEMP);
		rename("$Filename.tmp",$Filename);
	}
}

1;
