#!/usr/bin/perl

# "httpd.pl" in the Forum News Gateway
# copyright 1994,1995 by Swarthmore College

# Get "require" files from this directory, not from the perl library.
# This avoids possible name conflicts.
unshift (@INC,".");

require "common.ph";
require "config.pl";

require "caches.pl";
require "files.pl";
require "globals.pl";
require "httpdlib.pl";
require "killfile.pl";
require "newsrc.pl";
require "nntp.pl";
require "sendpages.pl";
require "tcp.pl";
require "threads.pl";
require "time.pl";

if (@ARGV < 1 || $ARGV[0] !~ /[0-9]+/) {
	print STDERR "usage: perl $0 <localportnum>\n",
				 "    where <localportnum> is the number of the port\n",
				 "    on which you want to listen for connections.\n";
	exit(1);
}

print STDERR "Starting up...\n";

$LocalHostName = "";		# will be set the first time we make a TCP connection somewhere
$LocalIPNum = "";			# will also be set the first time we make a TCP connection somewhere
$LocalPortNum = $ARGV[0];
$RemoteHostName = "";		# will be set each time we get a new client connection
$RemoteIPNum = "";			# will be set each time we get a new client connection

# "Re"-configure the gateway (actually, configure it for the first time).
exit(1) if (!&ReconfigureGateway());

# Initialize.
&InitGlobalVars();

print STDERR "Listening for connections.\n";

# These variables are used in checking for pending TCP connections, below.
$ReadIn = "";
$ReadOut = "";
vec($ReadIn,fileno(LISTENER),1) = 1;
$ExceptIn = $ReadIn;
$ExceptOut = "";
$ConnectionWaiting = 0;
$TimeLeft = 0;

$Done = 0;
while (!$Done) {
	# I had to give two arguments to this eval because with only one argument perl was
	# leaking memory badly.  I do not know why this works, but it fixes the leak.  I also
	# had to do this to the eval further down in this file where I handle exceptions. -- Asa
	eval(<<'MemoryLeakKludge',<<'EndOfEval');
MemoryLeakKludge
		
#		print STDERR "About to check for time left\n";
		
		# Check the socket to see whether a TCP connection is pending.
		# If no connection shows up for five minutes (300 seconds), do some lazy cache flushing.
		while (1) {
			($ConnectionWaiting,$TimeLeft) = select($ReadOut=$ReadIn,undef,$ExceptOut=$ExceptIn,300);
			last if ($ConnectionWaiting);
			last if (!&LazyUserCacheWrite());
		}
		
		&AcceptTCPConnection(CLIENT,LISTENER) || next;
		
#		print STDERR "About to read request\n";
		
		($Method,$Url,$ProtoVers,$Headers,$Body) = &ReadHTTPRequest();
		
		@UrlParts = &ParseURL($Url);
		if (@UrlParts == 0) {
			@UrlParts = ("WelcomePage");
		}
		
		$RemoteIPNum = &GetRemoteIPNum(CLIENT);
		if ($RemoteIPNum eq "") {
			print STDERR "Connection rejected: can't get remote IP number\n";
			&CloseTCPSocket(CLIENT);
			next;
		}
		
		($RemoteHostName,$Aliases,$AddrType,$Length,@Addrs) = gethostbyaddr($RemoteIPNum,&AF_INET);
		if ($RemoteHostName eq "") {
			local($A,$B,$C,$D) = unpack('C4',$RemoteIPNum);
			$RemoteHostName = $A.".".$B.".".$C.".".$D;
		}
		
#		print STDERR &ShortLocalTimeString(),", $RemoteHostName:\t",$UrlParts[0],"\t",$UrlParts[1],"\n";
		
		%Headers = ();
		%Headers = &BuildHeaderArray($Headers);
		
		# Parse form values if we got some.
		%FormValues = ();
		if (($Headers{"content-type"} =~ /application\/x\-www\-form\-urlencoded/i) || ($Method eq "GET" && $Body ne "")) {
			%FormValues = &BuildFormValuesArray($Body);
		}
		
		local($Realm) = "";
		if ($UrlParts[0] ne "") {
			$Realm = $UrlParts[0];
		}
		
		if (!&SiteAuthorized()) {
			if ($LogUnauth) {
				print STDERR "Unauthorized site $RemoteHostName requested $Method $Url\n";
			}
			&SendFileToClient($Url,$ProtoVers,"Unauthorized.html");
		} elsif ($UrlParts[0] eq "Files") {
			if ($LogSites) {
				print STDERR "Site $RemoteHostName\n";
			}
			if ($LogURLs) {
				print STDERR "$Method $Url\n";
			}
			
			local($Filename) = join('/',@UrlParts[1..$#UrlParts]);
			&SendFileToClient($Url,$ProtoVers,$Filename);
		} elsif (!&IsAuthorized($Headers{"authorization"},$Realm)) {
			if ($LogUnauth) {
				print STDERR "Authorization failed; site $RemoteHostName requested $Method $Url\n";
			}
			&SendAuthorizationRequiredMessage($ProtoVers,$Realm);
		} else {
			if ($LogSites) {
				print STDERR "Site $RemoteHostName\n";
			}
			if ($LogURLs) {
				print STDERR "$Method $Url\n";
			}
			
			if ($UrlParts[0] eq "WelcomePage") {
				&SendFileToClient($Url,$ProtoVers,"RootPage.html");
			} elsif ($UrlParts[0] eq "Administration") {
				&HandleAdministration();
			} elsif ($UrlParts[0] eq "AccountCreationPage") {
				&HandleUserAccountCreation();
			} elsif ($UrlParts[0] eq "Newsreading") {
				&HandleNewsreading();
			} elsif ($AllowPublicNewsreading && $UrlParts[0] eq "PublicNewsreading") {
				&HandlePublicNewsreading();
			} else {
				&SendHTTPNotFoundMessage($Url,$ProtoVers);
			}
		}
EndOfEval
	
#	print STDERR "cleaning up after handling request\n";
	
	if ($@ ne "" && $ExceptionCode == 0) {
		print STDERR "Unexpected Perl error in main eval: $@\n";
	}
	if ($ExceptionCode) {
		eval(<<'MemoryLeakKludge2',<<'EndOfEval2'); # At this point I will ignore any exceptions that are thrown
MemoryLeakKludge2
			if ($ExceptionCode == 503 || $ExceptionCode == 400) {
				# NNTP server timed out or disconnected us, so let's close our end of the connection.
				&SetCurrentNNTPServer("",0);
			}
			if ($ExceptionCode != $ERR_CLIENT_DIED) {
				&SendHTTPInternalErrorMessage($ProtoVers,$ExceptionCode,$ExceptionText);
			}
			$ExceptionCode = 0;
			$ExceptionText = "";
EndOfEval2
	}
	
	&CloseTCPSocket(CLIENT);
	
	if ($LogReply) {
		print STDERR "Sent response.\n";
	}
	$RemoteHostName = "";
	$RemoteIPNum = "";
}

&CleanupGlobalVars();

print STDERR "Normal shutdown.\n";

exit(0);


# ----------------------------
# Handle different kinds of requests.
# These routines are separated out to make the main loop more readable.
# Note that &HandleNewsreading is itself broken down into subroutines.
# These routines take no parameters. Their input is from global variables.

# For URLs of the form /Administration/...

sub HandleAdministration {
	if ($UrlParts[1] eq "Index") {
		&SendAdministrationPage($ProtoVers);
	} elsif ($UrlParts[1] eq "Reconfigure") {
		if (&ReconfigureGateway()) {
			&SendFileToClient($Url,$ProtoVers,"ReconfigOK.html");
		} else {
			&SendReconfigErrorPage($ProtoVers,"".$@);
			$@ = "";						# avoid double error reporting
			$ExceptionCode = 0;
		}
	} elsif ($UrlParts[1] eq "AccountCreationPage") {
		&HandleAccountCreation(1);
	} elsif ($UrlParts[1] eq "PasswordPage") {
		&HandlePasswordChange();
	} elsif ($UrlParts[1] eq "Quit") {
		&SendTerminationMessage($ProtoVers);
		$Done = 1;
	} else {
		&SendHTTPNotFoundMessage($Url,$ProtoVers);
	}
}


# For URLs of the form /AccountCreationPage.

sub HandleUserAccountCreation {
	if ($UserAccountCreation) {
		&HandleAccountCreation(0);
	} else {
		&SendHTTPNotFoundMessage($Url,$ProtoVers);
	}
}

# HandleAccountCreation($Admin)
# This routine does both administrator and user account creation.
# $Admin = 1 if the administrator is creating the account.
# $Admin = 0 if a user is creating the account.

sub HandleAccountCreation {
	local($Admin) = @_;
	
	if ($Method eq "POST") {
		local($ErrorMessage) = "";
		if ($FormValues{"username"} =~ /\W/) {
			$ErrorMessage .= "The user name can only contain alphanumeric characters. ";
		} 
		if (length($FormValues{"username"}) < $MinUserNameLength) {
			$ErrorMessage .= "The user name must be at least $MinUserNameLength characters long. ";
		}
		if (&DoesUserExist($FormValues{"username"})) {
			$ErrorMessage .= "User name \"".$FormValues{"username"}."\" is already taken. ";
		}
		if (length($FormValues{"password"}) < $MinPasswordLength) {
			$ErrorMessage .= "The password must be at least $MinPasswordLength characters long. ";
		}
		if ($FormValues{"password"} ne $FormValues{"confirmpassword"}) {
			$ErrorMessage .= "The password and the retyped password are different. ";
		}
		if ($FormValues{"realname"} eq "") {
			$ErrorMessage .= "The real name is missing. ";
		}
		if (!&ValidEmailAddress($FormValues{"email"})) {
			if ($FormValues{"email"} eq "") {
				$ErrorMessage .= "The e-mail address is missing. ";
			} else {
				$ErrorMessage .= "The e-mail address is incomplete. ";
			}
		}
		if ($ErrorMessage ne "") {
			&SendPrefsErrorPage($ProtoVers,"Could not create the account",$ErrorMessage,
					$Admin
						? "<A HREF=\"/Administration/AccountCreationPage\">Administrator Account Creation page</A>"
						: "<A HREF=\"/AccountCreationPage\">Account Creation page</A>");
		} else {
			# Create the account!
			&SetCurrentUser($FormValues{"username"});
			$Newsrc{"prefs","password"} = &NoDieCrypt($FormValues{"password"});
			$Newsrc{"prefs","realname"} = $FormValues{"realname"};
			$Newsrc{"prefs","email"} = $FormValues{"email"};
			$Newsrc{"prefs","organization"} = $FormValues{"organization"};
			$Newsrc{"prefs","markread"} = 0;
			$Newsrc{"prefs","hideempty"} = 0;
			$Newsrc{"prefs","nochecks"} = 0;
			$Newsrc{"time","created"} = "".time;
			$Newsrc{"","version"} = $CurrentNewsrcVersion;
			$KillFile{"","version"} = $CurrentKillFileVersion;
			
			# Write this info to disk immediately, so that &DoesUserExist
			# will know about the new user.
			&MyDBMFlush(*Newsrc,&NewsrcFilename($CurrentUser));
			&MyDBMFlush(*KillFile,&KillFilename($CurrentUser));
			
			if ($Admin) {
				&SetCurrentUser("root");
				&SendAccountCreationPage($ProtoVers,$Admin,"Account ".$FormValues{"username"}." created.");
			} else {
				&SendRedirectionMessage($ProtoVers,"http://$LocalHostName:$LocalPortNum/Newsreading/HomeNewsPage");
			}
		}
	} else {
		&SendAccountCreationPage($ProtoVers,$Admin,"");
	}
}

# For URLs of the form /Administration/PasswordPage.

sub HandlePasswordChange {
	if ($Method eq "POST") {
		local($ErrorMessage) = "";
		local($User) = $FormValues{"username"};
		if (!&DoesUserExist($User)) {
			$ErrorMessage .= "There is no such user account, \"$User\". ";
		}
		if (length($FormValues{"password"}) < $MinPasswordLength) {
			$ErrorMessage .= "The password must be at least $MinPasswordLength characters long. ";
		}
		if ($FormValues{"password"} ne $FormValues{"confirmpassword"}) {
			$ErrorMessage .= "The password and the retyped password are different. ";
		}
		if ($ErrorMessage ne "") {
			&SendPasswordPage($ProtoVers,$User,"<B>The password was not changed.</B> ".$ErrorMessage);
		} else {
			&SetCurrentUser($User);
			$Newsrc{"prefs","password"} = &NoDieCrypt($FormValues{"password"});
			$Newsrc{"","version"} = $CurrentNewsrcVersion;
			$KillFile{"","version"} = $CurrentKillFileVersion;
			&SendPasswordPage($ProtoVers,"","Password for \"$User\" changed.");
		}
	} else {
		&SendPasswordPage($ProtoVers,"","");
	}
}


# For URLs of the form /Newsreading/...

sub HandleNewsreading{
	if (($UrlParts[1] eq "NewsGroupPages") && (@UrlParts >= 4)) {
		&HandleNewsGroupPages(0);
	} elsif (($UrlParts[1] eq "ThreadPages") && (@UrlParts >= 5)) {
		&HandleThreadPages(0);
	} elsif (($UrlParts[1] eq "ArticlePages") && (@UrlParts >= 5)) {
		&HandleArticlePages(0);
	} elsif ($UrlParts[1] eq "HomeNewsPage") {
		&SendHomeNewsPage($ProtoVers);
	} elsif ($UrlParts[1] eq "SubscribePages") {
		&HandleSubscribePages();
	} elsif ($UrlParts[1] eq "PrefsPages") {
		&HandlePrefsPages();
	} elsif ($UrlParts[1] eq "AllGroupsPages") {
		&HandleAllGroupsPages();
	} elsif ($UrlParts[1] eq "KillPages") {
		&HandleKillPages();
	} elsif (($UrlParts[1] eq "TextArticlePages") && (@UrlParts >= 5)) {
		&HandleTextArticlePages(0);
	} elsif (($UrlParts[1] eq "PostArticlePages") && (@UrlParts >= 4)) {
		&HandlePostArticlePages(0);
	} else {
		&SendHTTPNotFoundMessage($Url,$ProtoVers);
	}
}


# For URLs of the form /PublicNewsreading/...

sub HandlePublicNewsreading{
	if (($UrlParts[1] eq "NewsGroupPages") && (@UrlParts >= 4)) {
		&HandleNewsGroupPages(1);
	} elsif (($UrlParts[1] eq "ThreadPages") && (@UrlParts >= 5)) {
		&HandleThreadPages(1);
	} elsif (($UrlParts[1] eq "ArticlePages") && (@UrlParts >= 5)) {
		&HandleArticlePages(1);
	} elsif (($UrlParts[1] eq "TextArticlePages") && (@UrlParts >= 5)) {
		&HandleTextArticlePages(1);
	} elsif ($AllowPublicPosting && ($UrlParts[1] eq "PostArticlePages") && (@UrlParts >= 4)) {
		&HandlePostArticlePages(1);
	} else {
		&SendHTTPNotFoundMessage($Url,$ProtoVers);
	}
}


# For URLs of the form /Newsreading/SubscribePages.

sub HandleSubscribePages {
	if ($Method eq "POST") {
		local($Text) = $FormValues{"groups"};
		local(@Lines) = split(/[\012\015]+/,$Text);
		local($Line,@Items,$Item);
		local(%HandledItems) = ();							# groups subscribed to
		local($Server);
		foreach $Server (@CanonicalNNTPServers) {			# for each news server
			&SetCurrentNNTPServer($Server,1);
			foreach $Line (@Lines) {						# for each line of text
				next if ($Line =~ /\!/);
				@Items = split(/[^a-zA-Z0-9_\.\-]+/,$Line);
				foreach $Item (@Items) {					# for each item on the line
					# Each item is a possible newsgroup.
					next if ($HandledItems{$Item});			# already got this one
					# If it's a newsgroup on this server, subscribe to it.
					if ($AllGroups{$Item} eq "/" || $AllGroups{$Item} =~ m#//#) {
						&SetCurrentGroup($Item);
						&SubscribeToGroup();
						$HandledItems{$Item} = "x";
					}
				}
			}
		}
		
		&SendRedirectionMessage($ProtoVers,"http://$LocalHostName:$LocalPortNum/Newsreading/HomeNewsPage");
	} else {
		&SendSubscribePage($ProtoVers);
	}
}

# For URLs of the form /Newsreading/PrefsPages.

sub HandlePrefsPages {
	if ($Method eq "POST") {
		local($ErrorMessage) = "";
		if ($FormValues{"password"} ne "" && length($FormValues{"password"}) < $MinPasswordLength) {
			$ErrorMessage .= "Your password must be at least $MinPasswordLength characters long. ";
		}
		if ($FormValues{"password"} ne $FormValues{"confirmpassword"}) {
			$ErrorMessage .= "Your password and your retyped password are different. ";
		}
		if ($FormValues{"realname"} eq "") {
			$ErrorMessage .= "You must type your real name. ";
		}
		if (!&ValidEmailAddress($FormValues{"email"})) {
			$ErrorMessage .= "You must type your complete e-mail address. ";
		}
		if ($ErrorMessage ne "") {
			&SendPrefsErrorPage($ProtoVers,"Made no change to your personal information",
					$ErrorMessage,
					"<A HREF=\"/Newsreading/PrefsPages\">Personal Information page</A>");
		} else {
			$Newsrc{"prefs","markread"} = ($FormValues{"markread"} eq "on");
			$Newsrc{"prefs","hideempty"} = ($FormValues{"hideempty"} eq "on");
			$Newsrc{"prefs","nochecks"} = ($FormValues{"usecheck"} ne "on");
			if ($FormValues{"password"} ne "") {
				$Newsrc{"prefs","password"} = &NoDieCrypt($FormValues{"password"});
			}
			$Newsrc{"prefs","realname"} = $FormValues{"realname"};
			$Newsrc{"prefs","email"} = $FormValues{"email"};
			$Newsrc{"prefs","organization"} = $FormValues{"organization"};
			$Newsrc{"prefs","signature"} = $FormValues{"signature"};
			$Newsrc{"time","created"} = "".time;		# '"".' makes sure the time is a string
			
			# Get the new signatures.
			# Convert newlines to \001 so the data can be written to the Newsrc file.
			# Change "." at the beginning of a line to ".." for the NNTP server.
			local($Sig);
			$Sig = $FormValues{"textsig"};
			$Sig =~ s/\001//g;				# first remove any real \001 characters
			$Sig =~ s/\015\012/\001/g;
			$Sig =~ s/\015/\001/g;
			$Sig =~ s/\012/\001/g;
			$Sig =~ s/(^|\001)\./$1\.\./g;
			$Newsrc{"prefs","textsig"} = $Sig;
			
			$Sig = $FormValues{"htmlsig"};
			$Sig =~ s/\001//g;				# first remove any real \001 characters
			$Sig =~ s/\015\012/\001/g;
			$Sig =~ s/\015/\001/g;
			$Sig =~ s/\012/\001/g;
			$Sig =~ s/(^|\001)\./$1\.\./g;
			$Newsrc{"prefs","htmlsig"} = $Sig;
			
			&MyDBMFlush(*Newsrc,&NewsrcFilename($CurrentUser));
			&SendRedirectionMessage($ProtoVers,"http://$LocalHostName:$LocalPortNum/Newsreading/HomeNewsPage");
		}
	} else {
		&SendPrefsPage($ProtoVers,"",0,"","","","","","","",0,0,1);
	}
}


# For URLs of the form /Newsreading/AllGroupsPages/...

sub HandleAllGroupsPages {
	local($NNTPServer) = &GetNNTPHostFromURL(@UrlParts);
	&SetCurrentNNTPServer($NNTPServer,1);
	
	local($GroupPrefix) = "";
	if (@UrlParts >= 4) {
		$GroupPrefix = join('.',@UrlParts[3..$#UrlParts]);
	}

	&SendAllGroupsPage($ProtoVers,$GroupPrefix);
}


# For URLs of the form /Newsreading/NewsGroupPages/..., if $PublicNewsreading is false
# For URLs of the form /PublicNewsreading/NewsGroupPages/..., if $PublicNewsreading is true

sub HandleNewsGroupPages {
	local($PublicNewsreading) = @_;
	
	local($NNTPServer) = &GetNNTPHostFromURL(@UrlParts);
	local($GroupName) = $UrlParts[3];
	
	if ($PublicNewsreading && !&AllowPublicNewsreadingOf($GroupName)) {
		&SendFileToClient($Url,$ProtoVers,"NoPublicReading.html");
		return;
	}
	
	local($Button) = $UrlParts[4];
	
	if ($Method eq "POST" && !$PublicNewsreading) {
		local($Action) = $FormValues{"Action"};
		
		&SetCurrentNNTPServer($NNTPServer,0);
		&SetCurrentGroup($GroupName,1);
		
		if ($Action eq "MarkAndGo") {
			&MarkGroupRead();
			local($Destination) = $FormValues{"Destination"};
			if ($Destination eq "home") {
				&SendHomeNewsPage($ProtoVers);
				return;		# do not continue on to &SendNewsGroupPage
			} else {
				($NNTPServer,$GroupName) = split(/ /,$Destination);
				# Continue on to &SendNewsGroupPage.
			}
		} elsif ($Action eq "SkipUnchecked") {
			&SkipUncheckedArticles();
		}
	}
	
	if ($Button ne "" && !$PublicNewsreading) {
		&SetCurrentNNTPServer($NNTPServer,0);
		&SetCurrentGroup($GroupName,1);
		if ($Button eq "MarkRead") {
			&MarkGroupRead();
		} elsif ($Button eq "MarkUnread") {
			&MarkGroupUnread();
		} elsif ($Button eq "Unsubscribe") {
			&UnsubscribeFromGroup();
		} elsif ($Button eq "Subscribe") {
			&SubscribeToGroup();
		}
	}
	&SendNewsGroupPage($ProtoVers,$NNTPServer,$GroupName,$PublicNewsreading);
	&RememberLastArticleInGroup();		# save high article number in %Newsrc, so we don't mark unseen articles read
}


# SkipUncheckedArticles()
# Read the form info to find out what threads in this group are
# unchecked, and mark them read.

sub SkipUncheckedArticles {
	local($Key,$Value);
	
	# The checkboxes have $Key = "T1", "T2", and so on, with $Value "on" or "".
	# There are corresponding hidden fields with parallel names $Key = "H1", "H2",
	# and so on, which have $Value equal to the thread's canonical subject.
	while (($Key,$Value) = each %FormValues) {
		if ($Key =~ /^H([0-9]+)$/ && $FormValues{"T".$1} ne "on") {
			&MarkThreadRead($Value);
		}
	}
}


# For URLs of the form /Newsreading/ThreadPages/... if $PublicNewsreading is false
# For URLs of the form /PublicNewsreading/ThreadPages/... if $PublicNewsreading is true

sub HandleThreadPages {
	local($PublicNewsreading) = @_;
	local($NNTPServer) = &GetNNTPHostFromURL(@UrlParts);
	local($GroupName) = $UrlParts[3];
	
	if ($PublicNewsreading && !&AllowPublicNewsreadingOf($GroupName)) {
		&SendFileToClient($Url,$ProtoVers,"NoPublicReading.html");
		return;
	}
	
	&SetCurrentNNTPServer($NNTPServer,0);
	&SetCurrentGroup($GroupName,1);
	local($Subject) = $UrlParts[4];

	&SendThreadPage($ProtoVers,$Subject,$PublicNewsreading);
}


# For URLs of the form /Newsreading/KillPages/...

sub HandleKillPages {
	if ($Method eq "POST") {
		local($Action) = $FormValues{"action"};
		
		if ($Action eq "highlights") {
			&RemoveCheckedHighlights();
		} elsif ($Action eq "kills") {
			&RemoveCheckedKills();
		} else {
			print STDERR "Unrecognized KillPages action.\n";
		}
	}
	
	&SendKillPage($ProtoVers);
}


# For URLs of the form /Newsreading/ArticlePages/... if $PublicNewsreading is false
# For URLs of the form /Newsreading/PublicArticlePages/... if $PublicNewsreading is true

sub HandleArticlePages {
	local($PublicNewsreading) = @_;
	
	local($NNTPServer) = &GetNNTPHostFromURL(@UrlParts);
	local($GroupName) = $UrlParts[3];
	
	if ($PublicNewsreading && !&AllowPublicNewsreadingOf($GroupName)) {
		&SendFileToClient($Url,$ProtoVers,"NoPublicReading.html");
		return;
	}
	
	&SetCurrentNNTPServer($NNTPServer,0);
	&SetCurrentGroup($GroupName,1);
	local($MessageID) = $UrlParts[4];
	
	if ($Method eq "POST" && !$PublicNewsreading) {
		local($Highlight) = $FormValues{"Highlight"};
		local($Kill) = $FormValues{"Kill"};
		local($ArticleNum) = $ThreadDB{$MessageID,"artnum"};
		
		if ($Highlight eq "Author") {
			&AddToKillFile($ArticleDB{$ArticleNum,"authorreal"},"highAuth");
		} elsif ($Highlight eq "Subject") {
			&AddToKillFile($ArticleDB{$ArticleNum,"subject"},"highSubj");
		} elsif ($Kill eq "Author") {
			&AddToKillFile($ArticleDB{$ArticleNum,"authorreal"},"killAuth");
		} elsif ($Kill eq "Subject") {
			&AddToKillFile($ArticleDB{$ArticleNum,"subject"},"killSubj");
		}
	}
	
	&SendArticlePage($ProtoVers,$MessageID,$PublicNewsreading);
}


# For URLs of the form /Newsreading/TextArticlePages/... if $PublicNewsreading is false
# For URLs of the form /PublicNewsreading/TextArticlePages/... if $PublicNewsreading is true

sub HandleTextArticlePages {
	local($PublicNewsreading) = @_;
	
	local($NNTPServer) = &GetNNTPHostFromURL(@UrlParts);
	local($GroupName) = $UrlParts[3];
	if ($PublicNewsreading && !&AllowPublicNewsreadingOf($GroupName)) {
		&SendFileToClient($Url,$ProtoVers,"NoPublicReading.html");
		return;
	}
	
	&SetCurrentNNTPServer($NNTPServer,0);
	&SetCurrentGroup($GroupName,1);
	
	&SendTextArticlePage($ProtoVers,$UrlParts[4],$PublicNewsreading);
}


# For URLs of the form /Newsreading/PostArticlePages/... if $PublicNewsreading is false
# For URLs of the form /PublicNewsreading/PostArticlePages/... if $PublicNewsreading is true

sub HandlePostArticlePages {
	local($PublicNewsreading) = @_;
	
	local($NNTPServer) = &GetNNTPHostFromURL(@UrlParts);
	local($GroupName) = $UrlParts[3];
	&SetCurrentNNTPServer($NNTPServer,0);
	&SetCurrentGroup($GroupName,1);
	local($MessageIDRespondingTo) = $UrlParts[4];
	
	if ($Method eq "POST") {
		local($Newsgroups) = $FormValues{"Newsgroups"};
		local($Subject) = $FormValues{"Subject"};
		local($Contenttype) = $FormValues{"Contenttype"};
		local($Body) = $FormValues{"Body"};
		local($Wrap) = $FormValues{"wraplines"} eq "on";
		local($Author,$EMail,$Organization);
		if ($PublicNewsreading) {
			$Author       = $FormValues{"Author"};
			$EMail        = $FormValues{"EMail"};
			$Organization = $FormValues{"Organization"};
		} else {
			$Author       = $Newsrc{"prefs","realname"};
			$EMail        = $Newsrc{"prefs","email"};
			$Organization = $Newsrc{"prefs","organization"};
		}
		
		if ($Wrap) {
			# Line-wrap the body of the message at column 70.
			# Existing line breaks are preserved and new ones added if necessary.
			# This can mess up if the body contains single words which are longer
			# than 70 characters--not an important case to worry about; they can
			# turn off wrapping. It also uglifies quoted lines which wrap; ick.
			local(@Paragraphs) = split(/\012/,$Body);	# loses trailing newlines
			local(@WrappedParagraphs) = ();
			local($Paragraph);
			local(@Words);
			local($Line,$Length);
			local(@WrappedLines);
			local($Word,$WordLength);
			local($WrappedParagraph);
			foreach $Paragraph (@Paragraphs) {
				# Handle ">" lines first; they are left unwrapped, so that quoted text
				# doesn't get mutilated by line breaks.
				if ($Paragraph =~ /^\>/) {
					push(@WrappedParagraphs,$Paragraph);
					next;
				}
				
				@Words = split(/ /,$Paragraph);			# loses trailing blanks
				$Line = "";
				$Length = -1;							# compensate for lack of blank character before first word
				@WrappedLines = ();
				foreach $Word (@Words) {
					$WordLength = 1 + length($Word);	# add one to length to account for the blank character
					if ($Length + $WordLength > 70 && $Line ne "") {
						push(@WrappedLines,$Line);
						$Line = $Word;
						$Length = $WordLength;
					} else {
						$Line .= " ".$Word;
						$Length += $WordLength;
					}
				}
				$WrappedParagraph = join("\012",@WrappedLines,$Line);
				# Snip out the initial blank that we may have added.
				$WrappedParagraph =~ s/^ //;
				push(@WrappedParagraphs,$WrappedParagraph);
			}
			$Body = join("\012",@WrappedParagraphs);
		}
		
		local($MIMEType) = "text/plain";
		if ($Contenttype eq "HTML") {
			$MIMEType = "text/html";
		} elsif ($Contenttype eq "TeX") {
			$MIMEType = "application/x-tex";
		}
		&SendPostArticleResult(
			$ProtoVers,$PublicNewsreading,
			$Author,$EMail,$Organization,$Newsgroups,$Subject,
			$MIMEType,$Body,$MessageIDRespondingTo
		);
	} else {
		local($IncludeOriginal) = $FormValues{"original"} eq "on";
		if ($PublicNewsreading) {
			&SendPublicPostArticlePage($ProtoVers,$MessageIDRespondingTo,$IncludeOriginal);
		} else {
			&SendPostArticlePage($ProtoVers,$MessageIDRespondingTo,$IncludeOriginal);
		}
	}
}
