# "reconfig.pl" in the Forum News Gateway
# copyright 1994,1995 by Swarthmore College

# ----- version

# This version string is inserted at the end of each user's news home page.
# You might use it to include, say, a link to a page of instructions.

$GatewayVersion = "<HR>Forum News Gateway 0.6 beta 7\012";


# ----- use XOVER NNTP command

# Some NNTP servers support the XOVER command for fetching article
# overview information. XOVER is much faster than the alternative
# of repeated HEAD and NEXT commands. Unfortunately, on at least one
# server the XOVER command is broken in that it will return information
# about articles which do not exist.

# If $UseXOVER is 1, the gateway will attempt to use XOVER. If
# this fails, it will fall back on HEAD and NEXT. The overhead of
# trying XOVER when it is not supported is usually slight.
# If $UseXOVER is 0, the gateway always uses HEAD and NEXT.

# Leave $UseXOVER set to 1 unless either:
# (1) You know that none of your NNTP servers uses XOVER, and
#     don't expect them to in the future, or
# (2) you suspect that XOVER may be broken on one of your NNTP servers.

# A possible symptom of broken XOVER is that newsgroups frequently
# contain articles which, when you click on them, turn out not to exist.
# Unfortunately, other problems can cause this symptom too.

$UseXOVER = 1;


# ----- rechecking articles

# The gateway caches 

$RecheckInterval = 12*60*60;


# ----- group filter pattern

# Newsgroups whose names match the group filter pattern, a perl regular
# expression, are not accessible to users. If the pattern is "", then all
# groups are allowed.

$GroupFilterPattern = "";

# Example. Disallow access to local groups with names starting "_":
# $GroupFilterPattern = "^_";


# ----- public newsreading

# The "public newsreading" feature allows users to read groups
# without having an account on the gateway. It's handy if you
# want to make discussions publicly available.
# Of course, without an account the gateway can't keep track of
# subscription information and remember what articles have been read.

# Set $AllowPublicNewsreading = 1 if you want public newsreading.
# It's turned off by default for compatibility with old versions.
# When it's turned on, the gateway recognizes URLs beginning
# /PublicNewsreading/NewsgroupPages/...
# /PublicNewsreading/ThreadPages/...
# /PublicNewsreading/ArticlePages/...
# /PublicNewsreading/TextArticlePages/...
# /PublicNewsreading/PostArticlePages/...
# There's no list of publicly available newsgroups. You should
# create links elsewhere to those you want to make available.
# You can restrict which groups are publicly available with
# &AllowPublicNewsreadingOf(), below.

$AllowPublicNewsreading = 0;

# If you want to allow anonymous users to post to public newsgroups,
# set $AllowPublicPosting = 1. Think carefully before turning this
# on. Unless your NNTP server carries only local groups, you probably
# want to restrict which groups allow anonymous public posting
# with &AllowPublicPostingTo(), below.

$AllowPublicPosting = 0;

# AllowPublicNewsreadingOf($Group)
# Return whether to allow public newsreading of the given group.
# This routine is called only when $AllowPublicNewreading is true.
# By default, reading of all groups is allowed. You can rewrite the
# routine to change this.

sub AllowPublicNewsreadingOf {
	return 1;
}

# Example. Allow reading only of groups beginning "local.".

# sub AllowPublicNewsreadingOf {
#	local($Group) = @_;
#	return $Group =~ /^local\./;
# }

# AllowPublicPostingTo($Group)
# Return whether to allow public posting to the given group.
# This routine is called only when $AllowPublicNewreading is true.
# By default, posting to all groups is allowed. You can rewrite the
# routine to change this.

sub AllowPublicPostingTo {
	return 1;
}

# Example. Allow posting only to "alt" groups.

# sub AllowPublicPostingTo {
#	local($Group) = @_;
#	return $Group =~ /^alt\./;
# }

# Example. Allow posting to "alt" groups, except
# for "alt.sex" groups.

# sub AllowPublicPostingTo {
#	local($Group) = @_;
#	return $Group =~ /^alt\./ && $Group !~ /^alt\.sex/;
# }


# Optional titles and other HTML stuff for public newsreading.
# Strings that you put into these variables are inserted in
# various places into the HTML newsgroup pages that the gateway
# returns for /PublicNewsreading/NewsgroupPages URLs.
# Access via a /Newsreading URL is not affected

# This string is inserted verbatim just before the end of each newsgroup page.
# It can contain any HTML that is allowed between <BODY> and </BODY>.
# You might use it to provide a link back to the gateway's welcome page.
$GlobalPublicTrailer = "<HR>";

# Each public newsgroup can be given its own title which overrides the
# usual newsgroup name.
# To call rec.games.backgammon instead "Backgammon", set
#   $PublicTitles{"rec.games.backgammon"} = "Backgammon";
# You can set as many titles as you like.
%PublicTitles = ();

# Each public newsgroup can be given its own HTML trailer, which
# is inserted after the newsgroup contents but before the
# global public trailer.
# The trailer can contain any HTML that is allowed between <BODY> and </BODY>.
# You might use it to describe the group or to provide links to
# more information.
# To make the rec.games.backgammon page end with "It's just a dice game.", set
#   $PublicTrailers{"rec.games.backgammon"} = "It's just a dice game.";
# You can set as many trailers as you like.
%PublicTrailers = ();


# ----- account creation and site authorization

# If users are to be able to create their own accounts, set
# $UserAccountCreation = 1. Anyone who is able to connect to the gateway
# will be able to create an account on it (possibly lying about their
# name and e-mail address and so on).

# If the administrator is to create all accounts, set
# $UserAccountCreation = 0.

# The administrator can always create accounts, whether users are allowed
# to or not.

$UserAccountCreation = 1;

# SiteAuthorized()
# This routine should return 1 if users on the connecting host are to be
# allowed to use the gateway; otherwise 0. If your site is not authorized,
# all you see is the site-not-authorized html page.
# By default, this always returns 1. Change it if you want to restrict usage.

# Here are two globals that the routine can refer to:
# $RemoteIPNum    - the IP number of the connecting machine
#                   (as a four-character string, in the usual Perl way)
# $RemoteHostName - a string, the name of the connecting machine
# The remote host name will look like "forum.swarthmore.edu" if we are able
# to get the machine's name; otherwise it will look like "130.58.72.2".

sub SiteAuthorized {
	return 1;
}

# Example. Restrict access to users connecting from 130.58.72.2:

#sub SiteAuthorized {
#	return $RemoteIPNum eq pack('C4',130,58,72,2);
#}

# Example. Restrict access to users in domain ".swarthmore.edu"
# whose machine name we can get:

#sub SiteAuthorized {
#	return $RemoteHostName =~ /\.swarthmore\.edu$/;
#}


# ----- page expiration

# The HTTP protocol provides an expiration time for HTML pages. Some web
# clients even pay attention to it: when a page expires it is purged from
# the client's cache, and refetched if it is again needed.

# These variables determine how long different pages take to expire. The
# default settings work fairly well; there's usually no need to change them.
# If you think users should have to explicitly reload expired pages, then
# make all the numbers big; it may cut the load on the server.

# The expiration times are in seconds. An expiration time of 0 should mean
# that the page expires immediately and should not be cached. Since most
# clients ignore the zero expirations, I've tried an expiration time of
# 1 second; maybe they'll pay attention to that.
# 60 seconds = one minute
# 60*60 seconds = one hour
# 24*60*60 seconds = 24 hours = one day

# Files, for /WelcomePage and /Files URLs.
$FileExpirationSeconds = 24*60*60;

# For /Administration, /AccountCreation, /Newsreading/PrefsPages,
# and /Newsreading/PostArticlePages URLs (which hardly ever change).
$BuiltInExpirationSeconds = 24*60*60;

# For the home news page, /Newsreading/HomeNewsPage.
$HomeExpirationSeconds = 1;

# For the highlights and kills page, /Newsreading/KillPages.
$KillExpirationSeconds = 1;

# For the newsgroup listings, /Newsreading/AllGroupsPages/.
$AllGroupsExpirationSeconds = 60*60;

# For a newsgroup's article listing, /Newsreading/NewsGroupPages, and
# the list of articles in a thread, /Newsreading/ThreadPages.
$NewsgroupExpirationSeconds = 60*60;

# For articles proper, /Newsreading/ArticlePages and
# /Newsreading/TextArticlePages.
$ArticleExpirationSeconds = 60*60;


# ----- serving different types of files via the /Files/... URL.

# To serve a file that a client requests via a /Files URL, the gateway
# needs to know what "Content-type:" header to give it. The content-type
# is a MIME type as defined in RFC 1521,
# <http://ds.internic.net/rfc/rfc1521.txt>.
# The gateway chooses a content-type based on the filename's suffix
# by calling the routine below, TranslateSuffixToType.

# This routine accepts a file name and returns a "Content-type:"
# header value. You can extend it to recognize more file suffixes
# by adding lines of the form
#	return "this/that" if ($Filename =~ /\.xyz$/i);
# If the file does not have a recognized suffix, it is assumed to
# be an HTML file. (So for simplicity the ".html" suffix is not
# recognized.)

sub TranslateSuffixToType {
	local($Filename) = @_;
	
	return "image/gif"  if ($Filename =~ /\.gif$/i);
	return "image/jpeg" if ($Filename =~ /\.jpg$/i || $Filename =~ /\.jpeg$/i);
	return "text/plain" if ($Filename =~ /\.txt$/i || $Filename =~ /\.text$/i);
	
	return "text/html";
}


# ----- logging

# The server outputs log information to standard error. By default, it logs
# errors and a few other tidbits. You can choose to log more information.

# To log changes of news server, news group, and user,
# set $LogChanges = 1.

$LogChanges = 0;

# To log the address of each connecting machine at each request,
# set $LogSites = 1.
# To log the URL of each request, set $LogURLs = 1.
# If you set them both to 1, you'll be able to see what URL each
# site requested.

$LogSites = 0;
$LogURLs = 0;

# To log unauthorized accesses, set $LogUnauth = 1.
# Both unauthorized sites and users who mistype their name or password
# will appear in the log, with site and URL information (but not password,
# of course).

$LogUnauth = 0;

# To log that a response has been sent to a request, set $LogReply = 1.
# This is handy for debugging.

$LogReply = 0;


# -- -- --


1;
