# "tcp.pl" in the Forum News Gateway
# copyright 1994,1995 by Swarthmore College

# Handle TCP protocol details.

require "sys/socket.ph";	# works for both perl4 and perl5
# use Socket;				# works for perl5, may be more reliable than the other way

# OpenPassiveTCPSocket($FileHandle,$Port)
# Start listening for TCP connections on the specified port using the given socket.
# This routine does not do accept().
# Return 1 if successful, 0 otherwise.

sub OpenPassiveTCPSocket {
	local($FileHandle,$Port) = @_;
	
	local($Name, $Aliases, $Proto) = getprotobyname('tcp');
	if ($Port !~ /^\d+$/) {
		($Name, $Aliases, $Port, $Proto) = getservbyname($Port, 'tcp');
	}
	
	local($This) = pack($TCPSockAddr, &AF_INET, $Port, "\0\0\0\0");
	
	socket($FileHandle, &AF_INET, &SOCK_STREAM, $Proto) || return 0;
	bind($FileHandle, $This) || return 0;
	listen($FileHandle, 5) || return 0;

	local($OldOutput) = select($FileHandle); $| = 1; select($OldOutput);
	
#	binmode($FileHandle);
	
	1;
}

# AcceptTCPConnection($NewSocket,$GenericSocket)
# calls accept() and then sets up filehandle-specific stuff for $NewSocket.

sub AcceptTCPConnection {
	local($NewSocket,$GenericSocket) = @_;
	local($Result);
	
	if ($Result = accept($NewSocket,$GenericSocket)) {
		binmode($NewSocket);
		local($OldOutput) = select($NewSocket); $| = 1; select($OldOutput);
	}
	return $Result;
}

# OpenActiveTCPSocket($FileHandle,$RemoteHost,$RemotePort)
# Connect the given TCP socket to the specified host and port.
# Return 1 for success, 0 for failure.

sub OpenActiveTCPSocket {
	local($FileHandle,$RemoteHost,$RemotePort) = @_;
	
	local($Name, $Aliases, $Proto) = getprotobyname('tcp');
	if ($RemotePort !~ /^\d+$/) {
		($Name, $Aliases, $RemotePort, $Proto) = getservbyname($RemotePort, 'tcp');
	}
	
	local($Type,$Len,$ThatAddr,$ThisAddr);
	($Name,$Aliases,$Type,$Len,$ThatAddr) = gethostbyname($RemoteHost);
	$ThisAddr = pack("C4",0,0,0,0);
	
	local($This) = pack($TCPSockAddr,&AF_INET,0, $ThisAddr);	# any port
	local($That) = pack($TCPSockAddr,&AF_INET,$RemotePort, $ThatAddr);
	
	socket($FileHandle,&AF_INET,&SOCK_STREAM, $Proto) || return 0;
	bind($FileHandle, $This) || return 0;
	connect($FileHandle,$That) || return 0;
	
	# set us up for unbuffered I/O, and binary mode.
	local($OldOutput) = select($FileHandle); $| = 1; select($OldOutput);
	
#	binmode($FileHandle);
	
	if ($LocalHostName eq "") {
		local($MySockAddr) = getsockname($FileHandle);
		local($Family,$Port,$MyAddr) = unpack($TCPSockAddr,$MySockAddr);
		
		$LocalIPNum = $MyAddr;
		local($Name,$Aliases,$AddrType,$Length,@Addrs) = gethostbyaddr($MyAddr,&AF_INET);
		$LocalHostName = $Name;		
#		print STDERR "Setting local host name to \"$Name\"\n";
	}
	
	1;
}

# GetRemoteIPNum($FileHandle)
# Return the IP number of the machine on the other end of the given TCP connection.
# Return "" if there's an error.

sub GetRemoteIPNum {
	local($FileHandle) = @_;
	
	local($That) = getpeername($FileHandle);
	return "" if ($That eq "");		# if getpeername failed
	
	local($Family,$Port,$RemoteAddr) = unpack($TCPSockAddr,$That);
	
	if ($Family != &AF_INET) {
		print STDERR "In GetRemoteIPAddr: socket is not an AF_INET socket!\n";
		return "";
	}
	
	return $RemoteAddr;
}

# CloseTCPSocket($FileHandle)
# Close the socket, returning an error code.

# If the CLIENT connection is already broken, perhaps because the
# client broke it to interrupt the transfer, then closing the CLIENT socket
# may fail. I found this to happen under rare conditions, and it
# probably depends on operating system details.

sub CloseTCPSocket {
	local($FileHandle) = @_;
	local($Result);
	
	$Result = eval 'close($FileHandle)';
#	if ($Result != 0) {
		# Just for information.
		# print STDERR "CloseTCPSocket: Unable to close: $Result, $!\n";
#	} else {
		# print STDERR "CloseTCPSocket succeeded!\n";
#	}
	return $Result;
}

1;
