# "time.pl" in the Forum News Gateway
# copyright 1994,1995 by Swarthmore College


%Months = ( "Jan", 1, "Feb", 2, "Mar", 3, "Apr", 4, "May", 5, "Jun", 6, 
			 "Jul", 7, "Aug", 8, "Sep", 9, "Oct", 10, "Nov", 11, "Dec", 12 );


# TimeString($Time)
# Returns a string representing the given Greenwich Mean time, in
# a format acceptable for use in MIME Date: headers.

sub TimeString {
	local($Time) = @_;
	local($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime($Time);
	
	local(@Months) = (Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec);
	local(@Days) = (Sun,Mon,Tue,Wed,Thu,Fri,Sat);
	
	sprintf("%s %s %d %02d:%02d:%02d %04d GMT", $Days[$wday], $Months[$mon], $mday, $hour, $min, $sec, $year+1900);
}


# TimeStringToNNTPTimeString($TimeString)
# This routine converts a time string of the format returned by the TimeString routine
# into a string of the format required by the NEWGROUPS NNTP command.

sub TimeStringToNNTPTimeString {
	local($TimeString) = @_;
	
	if ($TimeString !~ /^\S+\s+(\S+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s+(\d+)/) {
		print STDERR "In TimeStringToNNTPTimeString: input \"$TimeString\" is not in the right format";
		return "000000 000000";
	}
	
	local($Month,$DayOfMonth,$Hour,$Minute,$Second,$Year) = ($1,$2,$3,$4,$5,$6);
	sprintf("%02d%02d%02d %02d%02d%02d GMT",$Year % 100, $Months{$Month}, $DayOfMonth, $Hour, $Minute, $Second);
}


# ShortTimeString($Time)
# Return a string representing the given time, in a compact format.
# If the time is "", return something non-timelike (spacelike), such as "-".

sub ShortTimeString {
	local($Time) = @_;
	
	return "-" if ($Time eq "");
	
	local($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($Time);
	
	local(@Months) = (Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec);
	local(@Days) = (Sun,Mon,Tue,Wed,Thu,Fri,Sat);

	sprintf("%s %d %02d:%02d:%02d %04d", $Months[$mon], $mday, $hour, $min, $sec, $year+1900);
}


# ShortLocalTimeString()
# Return a string representing the current local time, in
# a compact format for use in the log file.

sub ShortLocalTimeString {
	&ShortTimeString(time);
}

1;
